@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2019, the cclib development team
+# Copyright (c) 2020, the cclib development team
 #
 # This file is part of cclib (http://cclib.github.io) and is distributed under
 # the terms of the BSD 3-Clause License.
@@ -17,6 +17,8 @@ import os
 import random
 import sys
 import zipfile
+from abc import ABCMeta, abstractmethod
+from six import add_metaclass
 
 if sys.version_info.major == 2:
     getargspec = inspect.getargspec
@@ -95,10 +97,13 @@ class FileWrapper(object):
 
         # Assume the position is what was passed to the constructor.
         self.pos = pos
+        
+        self.last_line = None
 
     def next(self):
         line = next(self.src)
         self.pos += len(line)
+        self.last_line = line
         return line
 
     def __next__(self):
@@ -182,6 +187,7 @@ def openlogfile(filename, object=None):
         return fileobject
 
 
+@add_metaclass(ABCMeta)
 class Logfile(object):
     """Abstract class for logfile objects.
 
@@ -190,8 +196,8 @@ class Logfile(object):
         NWChem, ORCA, Psi, Q-Chem
     """
 
-    def __init__(self, source, loglevel=logging.INFO, logname="Log",
-                 logstream=sys.stdout, datatype=ccData_optdone_bool, **kwds):
+    def __init__(self, source, loglevel=logging.ERROR, logname="Log",
+                 logstream=sys.stderr, datatype=ccData_optdone_bool, **kwds):
         """Initialise the Logfile object.
 
         This should be called by a subclass in its own __init__ method.
@@ -330,6 +336,10 @@ class Logfile(object):
             except StopIteration:
                 self.logger.error("Unexpectedly encountered end of logfile.")
                 break
+            except Exception as e:
+                self.logger.error("Encountered error when parsing.")
+                self.logger.error("Last line read: %s" % inputfile.last_line)
+                raise
 
         # Close input file object.
         if not self.isstream:
@@ -372,6 +382,9 @@ class Logfile(object):
             if not attr in _nodelete:
                 self.__delattr__(attr)
 
+        # Perform final checks on values of attributes.
+        data.check_values(logger=self.logger)
+
         # Update self.progress as done.
         if hasattr(self, "progress"):
             self.progress.update(inputfile.size, "Done")
@@ -395,28 +408,9 @@ class Logfile(object):
                 self.progress.update(newstep, msg)
                 self.progress.step = newstep
 
+    @abstractmethod
     def normalisesym(self, symlabel):
-        """Standardise the symmetry labels between parsers.
-
-        This method should be overwritten by individual parsers, and should
-        contain appropriate doctests. If is not overwritten, this is detected
-        as an error by unit tests.
-        """
-        raise NotImplementedError("normalisesym(self, symlabel) must be overriden by the parser.")
-
-    def float(self, number):
-        """Convert a string to a float.
-
-        This method should perform certain checks that are specific to cclib,
-        including avoiding the problem with Ds instead of Es in scientific notation.
-        Another point is converting string signifying numerical problems (*****)
-        to something we can manage (Numpy's NaN).
-        """
-
-        if list(set(number)) == ['*']:
-            return numpy.nan
-
-        return float(number.replace("D", "E"))
+        """Standardise the symmetry labels between parsers."""
 
     def new_internal_job(self):
         """Delete attributes that can be problematic in multistep jobs.
@@ -434,19 +428,28 @@ class Logfile(object):
             if hasattr(self, name):
                 delattr(self, name)
 
-    def set_attribute(self, name, value, check=True):
-        """Set an attribute and perform a check when it already exists.
+    def set_attribute(self, name, value, check_change=True):
+        """Set an attribute and perform an optional check when it already exists.
 
         Note that this can be used for scalars and lists alike, whenever we want
-        to set a value for an attribute. By default we want to check that
-        the value does not change if the attribute already exists, and this function
-        is a good place to add more tests in the future.
+        to set a value for an attribute.
+        
+        Parameters
+        ----------
+        name: str
+            The name of the attribute.
+        value: str
+            The value for the attribute.
+        check_change: bool
+            By default we want to check that the value does not change
+            if the attribute already exists.
         """
-        if check and hasattr(self, name):
+        if check_change and hasattr(self, name):
             try:
                 numpy.testing.assert_equal(getattr(self, name), value)
             except AssertionError:
                 self.logger.warning("Attribute %s changed value (%s -> %s)" % (name, getattr(self, name), value))
+
         setattr(self, name, value)
 
     def append_attribute(self, name, value):
@@ -456,6 +459,13 @@ class Logfile(object):
             self.set_attribute(name, [])
         getattr(self, name).append(value)
 
+    def extend_attribute(self, name, values):
+        """Appends an iterable of values to an attribute."""
+        
+        if not hasattr(self, name):
+            self.set_attribute(name, [])
+        getattr(self, name).extend(values)
+
     def _assign_coreelectrons_to_element(self, element, ncore,
                                          ncore_is_total_count=False):
         """Assign core electrons to all instances of the element.
