@@ -265,9 +265,24 @@ class Molpro(logfileparser.Logfile):
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
 
-        # Extract the package version number.
+        # Extract the package version number. The one associated with NAME may
+        # be more specific.
+        if line.strip()[:4] == "NAME":
+            self.metadata["package_version"] = line.split()[-1]
+        # Only take the less-specific version if it doesn't already exist.
         if "Version" in line:
-            self.metadata["package_version"] = line.split()[1]
+            package_version = self.metadata.get("package_version")
+            less_specific_package_version = line.split()[1]
+            if not package_version:
+                self.metadata["package_version"] = less_specific_package_version
+            # ...but use it for the legacy (short) version.
+            self.metadata["legacy_package_version"] = less_specific_package_version
+        if "SHA1" in line:
+            package_version = self.metadata.get("package_version")
+            if package_version:
+                self.metadata["package_version"] = ''.join([
+                    package_version, "+", line.split()[-1]
+                ])
 
         if line[1:19] == "ATOMIC COORDINATES":
 
@@ -476,6 +491,11 @@ class Molpro(logfileparser.Logfile):
                     break
             self.scfvalues.append(numpy.array(scfvalues))
 
+        if "dispersion correction" in line \
+           and line.strip() != "dispersion correction activated":
+            dispersion = utils.convertor(float(line.split()[-1]), "hartree", "eV")
+            self.append_attribute("dispersionenergies", dispersion)
+
         # SCF result - RHF/UHF and DFT (RKS) energies.
         if (line[1:5] in ["!RHF", "!UHF", "!RKS"] and line[16:22].lower() == "energy"):
 
@@ -533,6 +553,8 @@ class Molpro(logfileparser.Logfile):
             if not hasattr(self, "ccenergies"):
                 self.ccenergies = []
             while line[1:20] != "Program statistics:":
+                if line[71:84] == "T1 diagnostic":
+                    self.metadata["t1_diagnostic"] = float(line.split()[-1])
                 # The last energy (most exact) will be read last and thus saved.
                 if line[1:5] == "!CCD" or line[1:6] == "!CCSD" or line[1:9] == "!CCSD(T)":
                     ccenergy = float(line.split()[-1])
