@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2019, the cclib development team
+# Copyright (c) 2020, the cclib development team
 #
 # This file is part of cclib (http://cclib.github.io) and is distributed under
 # the terms of the BSD 3-Clause License.
@@ -10,9 +10,13 @@
 
 from __future__ import print_function
 
-import numpy
 import re
 
+import numpy
+from packaging.version import parse as parse_version
+
+from six.moves import zip_longest
+
 from cclib.parser import logfileparser
 from cclib.parser import utils
 
@@ -25,7 +29,6 @@ class ORCA(logfileparser.Logfile):
         # Call the __init__ method of the superclass
         super(ORCA, self).__init__(logname="ORCA", *args, **kwargs)
 
-
     def __str__(self):
         """Return a string representation of the object."""
         return "ORCA log file %s" % (self.filename)
@@ -47,12 +50,49 @@ class ORCA(logfileparser.Logfile):
         # Keep track of whether this is a relaxed scan calculation
         self.is_relaxed_scan = False
 
+    def after_parsing(self):
+        # ORCA doesn't add the dispersion energy to the "Total energy" (which
+        # we parse), only to the "FINAL SINGLE POINT ENERGY" (which we don't
+        # parse).
+        if hasattr(self, "scfenergies") and hasattr(self, "dispersionenergies"):
+            for i, (scfenergy, dispersionenergy) in enumerate(
+                zip_longest(self.scfenergies, self.dispersionenergies)
+            ):
+                # It isn't as problematic if there are more dispersion than
+                # SCF energies, since all dispersion energies can still be
+                # added to the SCF energies, hence the difference in log level.
+                if dispersionenergy is None:
+                    self.logger.error(
+                        "The number of SCF and dispersion energies are not equal: %d vs. %d, "
+                        "can't add dispersion energy to all SCF energies",
+                        len(self.scfenergies),
+                        len(self.dispersionenergies)
+                    )
+                    break
+                if scfenergy is None:
+                    self.logger.warning(
+                        "The number of SCF and dispersion energies are not equal: %d vs. %d, "
+                        "can't add dispersion energy to all SCF energies",
+                        len(self.scfenergies),
+                        len(self.dispersionenergies)
+                    )
+                    break
+                self.scfenergies[i] += dispersionenergy
+
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
 
         # Extract the version number.
         if "Program Version" == line.strip()[:15]:
-            self.metadata["package_version"] = line.split()[2]
+            # Handle development versions.
+            self.metadata["legacy_package_version"] = line.split()[2]
+            self.metadata["package_version"] = self.metadata["legacy_package_version"].replace(".x", "dev")
+            possible_revision_line = next(inputfile)
+            if "SVN: $Rev" in possible_revision_line:
+                self.metadata["package_version"] += "+{}".format(
+                    re.search(r"\d+", possible_revision_line).group()
+                )
+
 
         # ================================================================================
         #                                         WARNINGS
@@ -173,7 +213,7 @@ class ORCA(logfileparser.Logfile):
                                 continue
                             if line[0] == '#' or line.strip(' ') == '\n':
                                 continue
-                            if line[0] == '*' or line.strip() == "end":
+                            if line.strip()[0] == '*' or line.strip() == "end":
                                 break
                             # Strip basis specification that can appear after coordinates
                             line = line.split('newGTO')[0].strip()
@@ -283,6 +323,47 @@ class ORCA(logfileparser.Logfile):
 
             self._append_scfvalues_scftargets(inputfile, line)
 
+        """
+-------------------------------------------------------------------------------
+                          DFT DISPERSION CORRECTION
+
+                              DFTD3 V3.1  Rev 1
+                              USING zero damping
+-------------------------------------------------------------------------------
+The omegaB97X-D3 functional is recognized. Fit by Chai et al.
+Active option DFTDOPT                   ...         3
+
+molecular C6(AA) [au] = 9563.878941
+
+
+            DFT-D V3
+ parameters
+ s6 scaling factor         :     1.0000
+ rs6 scaling factor        :     1.2810
+ s8 scaling factor         :     1.0000
+ rs8 scaling factor        :     1.0940
+ Damping factor alpha6     :    14.0000
+ Damping factor alpha8     :    16.0000
+ ad hoc parameters k1-k3   :    16.0000     1.3333    -4.0000
+
+ Edisp/kcal,au: -10.165629059768  -0.016199959356
+ E6   /kcal   :  -4.994512983
+ E8   /kcal   :  -5.171116077
+ % E8         :  50.868628459
+
+-------------------------   ----------------
+Dispersion correction           -0.016199959
+-------------------------   ----------------
+"""
+        if "DFT DISPERSION CORRECTION" in line:
+            # A bunch of parameters are printed the first time dispersion is called
+            # However, they vary wildly in form and number, making parsing problematic
+            line = next(inputfile)
+            while 'Dispersion correction' not in line:
+                line = next(inputfile)
+            dispersion = utils.convertor(float(line.split()[-1]), "hartree", "eV")
+            self.append_attribute("dispersionenergies", dispersion)
+
         # The convergence targets for geometry optimizations are printed at the
         # beginning of the output, although the order and their description is
         # different than later on. So, try to standardize the names of the criteria
@@ -370,6 +451,79 @@ class ORCA(logfileparser.Logfile):
                 target = float(line.split()[-2])
                 self.geotargets_names.append(name)
                 self.geotargets.append(target)
+        
+
+        # Moller-Plesset energies.
+        #
+        # ---------------------------------------
+        # MP2 TOTAL ENERGY:      -76.112119693 Eh
+        # ---------------------------------------
+        if 'MP2 TOTAL ENERGY' in line[:16]:
+
+            if not hasattr(self, 'mpenergies'):
+                self.metadata['methods'].append('MP2')
+                self.mpenergies = []
+
+            self.mpenergies.append([])
+            mp2energy = utils.float(line.split()[-2])
+            self.mpenergies[-1].append(utils.convertor(mp2energy, 'hartree', 'eV'))
+
+        # MP2 energy output line is different for MP3, since it uses the MDCI
+        # code, which is also in charge of coupled cluster.
+        #
+        # MP3 calculation:
+        # E(MP2)  =    -76.112119775   EC(MP2)=    -0.128216417
+        # E(MP3)  =    -76.113783480   EC(MP3)=    -0.129880122  E3=    -0.001663705
+        #
+        # CCSD calculation:
+        # E(MP2)                                     ...     -0.393722942
+        # Initial E(tot)                             ...  -1639.631576169
+        # <T|T>                                      ...      0.087231847
+        # Number of pairs included                   ... 55
+        # Total number of pairs                      ... 55
+        if 'E(MP2)' in line:
+
+            if not hasattr(self, 'mpenergies'):
+                self.mpenergies = []
+
+            self.mpenergies.append([])
+            mp2energy = utils.float(line.split()[-1])
+            self.mpenergies[-1].append(utils.convertor(mp2energy, 'hartree', 'eV'))
+
+            line = next(inputfile)
+            if line[:6] == 'E(MP3)':
+                self.metadata['methods'].append('MP3')
+                mp3energy = utils.float(line.split()[2])
+                self.mpenergies[-1].append(utils.convertor(mp3energy, 'hartree', 'eV'))
+            else:
+                assert line[:14] == 'Initial E(tot)'
+
+        # ----------------------
+        # COUPLED CLUSTER ENERGY
+        # ----------------------
+        #
+        # E(0)                                       ...  -1639.237853227
+        # E(CORR)                                    ...     -0.360153516
+        # E(TOT)                                     ...  -1639.598006742
+        # Singles Norm <S|S>**1/2                    ...      0.176406354  
+        # T1 diagnostic                              ...      0.039445660  
+        if line[:22] == 'COUPLED CLUSTER ENERGY':
+            self.skip_lines(inputfile, ['d', 'b'])
+            line = next(inputfile)
+            assert line[:4] == 'E(0)'
+            scfenergy = utils.convertor(utils.float(line.split()[-1]), 'hartree', 'eV')
+            line = next(inputfile)
+            assert line[:7] == 'E(CORR)'
+            while 'E(TOT)' not in line:
+                line = next(inputfile)
+            self.append_attribute(
+                'ccenergies',
+                utils.convertor(utils.float(line.split()[-1]), 'hartree', 'eV')
+            )
+            line = next(inputfile)
+            assert line[:23] == 'Singles Norm <S|S>**1/2'
+            line = next(inputfile)
+            self.metadata["t1_diagnostic"] = utils.float(line.split()[-1])
 
         # ------------------
         # CARTESIAN GRADIENT
@@ -378,22 +532,37 @@ class ORCA(logfileparser.Logfile):
         # 1   H   :    0.000000004    0.019501450   -0.021537091
         # 2   O   :    0.000000054   -0.042431648    0.042431420
         # 3   H   :    0.000000004    0.021537179   -0.019501388
-        if line[:18] == 'CARTESIAN GRADIENT':
-            next(inputfile)
-            next(inputfile)
+        #
+        # ORCA MP2 module has different signal than 'CARTESIAN GRADIENT'.
+        #
+        # The final MP2 gradient
+        # 0:   0.01527469  -0.00292883   0.01125000
+        # 1:   0.00098782  -0.00040549   0.00196825
+        # 2:  -0.01626251   0.00333431  -0.01321825
+        if line[:18] == 'CARTESIAN GRADIENT' or line[:22] == 'The final MP2 gradient':
 
             grads = []
-            line = next(inputfile).strip()
-            while line:
-                idx, atom, colon, x, y, z = line.split()
-                grads.append((float(x), float(y), float(z)))
+            if line[:18] == 'CARTESIAN GRADIENT':
+                self.skip_lines(inputfile, ['dashes', 'blank'])
 
+            line = next(inputfile).strip()
+            if 'CONSTRAINED CARTESIAN COORDINATES' in line:
+                self.skip_line(
+                    inputfile, 'constrained Cartesian coordinate warning'
+                )
+                line = next(inputfile).strip()
+
+            while line:
+                tokens = line.split()
+                x, y, z = float(tokens[-3]), float(tokens[-2]), float(tokens[-1])
+                grads.append((x, y, z))
                 line = next(inputfile).strip()
 
             if not hasattr(self, 'grads'):
                 self.grads = []
             self.grads.append(grads)
 
+
         # After each geometry optimization step, ORCA prints the current convergence
         # parameters and the targets (again), so it is a good idea to check that they
         # have not changed. Note that the order of these criteria here are different
@@ -648,7 +817,7 @@ class ORCA(logfileparser.Logfile):
 
                         # This regex will tease out all number with exactly
                         # six digits after the decimal point.
-                        coeffs = re.findall('-?\d+\.\d{6}', line)
+                        coeffs = re.findall(r'-?\d+\.\d{6}', line)
 
                         # Something is very wrong if this does not hold.
                         assert len(coeffs) <= 6
@@ -708,15 +877,39 @@ class ORCA(logfileparser.Logfile):
                 self.gbasis.append(gbasis_tmp[bas_atname])
             del self.tmp_atnames
 
-        """ Banner announcing Thermochemistry
+        """
         --------------------------
         THERMOCHEMISTRY AT 298.15K
         --------------------------
-        """
-        if 'THERMOCHEMISTRY AT' == line[:18]:
 
-            next(inputfile)
-            next(inputfile)
+        Temperature         ... 298.15 K
+        Pressure            ... 1.00 atm
+        Total Mass          ... 130.19 AMU
+
+        Throughout the following assumptions are being made:
+          (1) The electronic state is orbitally nondegenerate
+          ...
+
+        freq.      45.75  E(vib)   ...       0.53 
+        freq.      78.40  E(vib)   ...       0.49
+        ...
+
+
+        ------------
+        INNER ENERGY
+        ------------
+
+        The inner energy is: U= E(el) + E(ZPE) + E(vib) + E(rot) + E(trans)
+             E(el)   - is the total energy from the electronic structure calc
+             ...
+
+        Summary of contributions to the inner energy U:
+        Electronic energy                ...   -382.05075804 Eh
+        ...
+        """
+        if line.strip().startswith('THERMOCHEMISTRY AT'):
+
+            self.skip_lines(inputfile, ['dashes', 'blank'])
             self.temperature = float(next(inputfile).split()[2])
             self.pressure = float(next(inputfile).split()[2])
             total_mass = float(next(inputfile).split()[3])
@@ -725,36 +918,52 @@ class ORCA(logfileparser.Logfile):
             line = next(inputfile)
             while line[:17] != 'Electronic energy':
                 line = next(inputfile)
-            self.zpe = next(inputfile).split()[4]
+            self.electronic_energy = float(line.split()[3])
+            self.zpe = float(next(inputfile).split()[4])
             thermal_vibrational_correction = float(next(inputfile).split()[4])
             thermal_rotional_correction = float(next(inputfile).split()[4])
             thermal_translational_correction = float(next(inputfile).split()[4])
-            next(inputfile)
+            self.skip_lines(inputfile, ['dashes'])
             total_thermal_energy = float(next(inputfile).split()[3])
 
             # Enthalpy
-            line = next(inputfile)
             while line[:17] != 'Total free energy':
                 line = next(inputfile)
             thermal_enthalpy_correction = float(next(inputfile).split()[4])
             next(inputfile)
-            self.enthalpy = float(next(inputfile).split()[3])
+
+            # For a single atom, ORCA provides the total free energy or inner energy
+            # which includes a spurious vibrational correction (see #817 for details).
+            if self.natom > 1:
+                self.enthalpy = float(next(inputfile).split()[3])
+            else:
+                self.enthalpy = self.electronic_energy + thermal_translational_correction
 
             # Entropy
-            line = next(inputfile)
             while line[:18] != 'Electronic entropy':
                 line = next(inputfile)
             electronic_entropy = float(line.split()[3])
             vibrational_entropy = float(next(inputfile).split()[3])
             rotational_entropy = float(next(inputfile).split()[3])
             translational_entropy = float(next(inputfile).split()[3])
-            next(inputfile)
-            self.entropy = float(next(inputfile).split()[4])
+            self.skip_lines(inputfile, ['dashes'])
 
-            line = next(inputfile)
-            while line[:25] != 'Final Gibbs free enthalpy':
+            # ORCA prints -inf for single atom entropy.
+            if self.natom > 1:
+                self.entropy = float(next(inputfile).split()[4])
+            else:
+                self.entropy = (electronic_entropy + translational_entropy) / self.temperature
+
+            while (line[:25] != 'Final Gibbs free enthalpy') and (line[:23] != 'Final Gibbs free energy'):
                 line = next(inputfile)
-            self.freeenergy = float(line.split()[5])
+            self.skip_lines(inputfile, ['dashes'])
+
+
+            # ORCA prints -inf for sinle atom free energy.
+            if self.natom > 1:
+                self.freeenergy = float(line.split()[5])
+            else:
+                self.freeenergy = self.enthalpy - self.temperature * self.entropy
 
         # Read TDDFT information
         if any(x in line for x in ("TD-DFT/TDA EXCITED", "TD-DFT EXCITED")):
@@ -930,7 +1139,8 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
                     return energy, intensity
 
             # Clashes with Orca 2.6 (and presumably before) TDDFT absorption spectrum printing
-            elif line == 'ABSORPTION SPECTRUM' and float(self.metadata['package_version']) > 2.6:
+            elif line == 'ABSORPTION SPECTRUM' and \
+                 parse_version(self.metadata['package_version']).release > (2, 6):
                 def energy_intensity(line):
                     """ CASSCF absorption spectrum
 ------------------------------------------------------------------------------------------
@@ -941,7 +1151,7 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
 ------------------------------------------------------------------------------------------
   0( 0)-> 1( 0) 1   83163.2    120.2   0.088250385   2.25340   0.00000   0.00000   1.50113
 """
-                    reg = r'(\d+)\( ?(\d+)\)-> ?(\d+)\( ?(\d+)\) (\d+)'+ '\s+(\d+\.\d+)'*4 + '\s+(-?\d+\.\d+)'*3
+                    reg = r'(\d+)\( ?(\d+)\)-> ?(\d+)\( ?(\d+)\) (\d+)'+ r'\s+(\d+\.\d+)'*4 + r'\s+(-?\d+\.\d+)'*3
                     res = re.search(reg, line)
                     jstate, jblock, istate, iblock, mult, energy, wavelength, intensity, t2, tx, ty, tz = res.groups()
                     return energy, intensity
@@ -986,8 +1196,8 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
                 if "spin forbidden" in line:
                     etrotat, mx, my, mz = 0.0, 0.0, 0.0, 0.0
                 else:
-                    etrotat, mx, my, mz = [self.float(t) for t in tokens[3:]]
-                etenergies.append(self.float(tokens[1]))
+                    etrotat, mx, my, mz = [utils.float(t) for t in tokens[3:]]
+                etenergies.append(utils.float(tokens[1]))
                 etrotats.append(etrotat)
                 line = next(inputfile)
             self.set_attribute("etrotats", etrotats)
@@ -1003,20 +1213,24 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
             if float(self.metadata["package_version"][:3]) > 4.0:
                 self.skip_lines(inputfile, ['Scaling factor for frequencies', 'b'])
 
-            vibfreqs = numpy.zeros(3 * self.natom)
-            for i, line in zip(range(3 * self.natom), inputfile):
-                vibfreqs[i] = float(line.split()[1])
+            if self.natom > 1:
+                vibfreqs = numpy.zeros(3 * self.natom)
+                for i, line in zip(range(3 * self.natom), inputfile):
+                    vibfreqs[i] = float(line.split()[1])
 
-            nonzero = numpy.nonzero(vibfreqs)[0]
-            self.first_mode = nonzero[0]
-            # Take all modes after first
-            # Mode between imaginary and real modes could be 0
-            self.num_modes = 3*self.natom - self.first_mode
-            if self.num_modes > 3*self.natom - 6:
-                msg = "Modes corresponding to rotations/translations may be non-zero."
-                if self.num_modes == 3*self.natom - 5:
-                    msg += '\n You can ignore this if the molecule is linear.'
-            self.set_attribute('vibfreqs', vibfreqs[self.first_mode:])
+                nonzero = numpy.nonzero(vibfreqs)[0]
+                self.first_mode = nonzero[0]
+                # Take all modes after first
+                # Mode between imaginary and real modes could be 0
+                self.num_modes = 3*self.natom - self.first_mode
+                if self.num_modes > 3*self.natom - 6:
+                    msg = "Modes corresponding to rotations/translations may be non-zero."
+                    if self.num_modes == 3*self.natom - 5:
+                        msg += '\n You can ignore this if the molecule is linear.'
+                self.set_attribute('vibfreqs', vibfreqs[self.first_mode:])
+            else:
+                # we have a single atom
+                self.set_attribute('vibfreqs', numpy.array([]))
 
         # NORMAL MODES
         # ------------
@@ -1031,18 +1245,22 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
         #       2       0.000000   0.000000   0.000000   0.000000   0.000000   0.000000
         # ...
         if line[:12] == "NORMAL MODES":
-            all_vibdisps = numpy.zeros((3 * self.natom, self.natom, 3), "d")
+            if self.natom > 1:
+                all_vibdisps = numpy.zeros((3 * self.natom, self.natom, 3), "d")
 
-            self.skip_lines(inputfile, ['d', 'b', 'text', 'text', 'text', 'b'])
+                self.skip_lines(inputfile, ['d', 'b', 'text', 'text', 'text', 'b'])
 
-            for mode in range(0, 3 * self.natom, 6):
-                header = next(inputfile)
-                for atom in range(self.natom):
-                    all_vibdisps[mode:mode + 6, atom, 0] = next(inputfile).split()[1:]
-                    all_vibdisps[mode:mode + 6, atom, 1] = next(inputfile).split()[1:]
-                    all_vibdisps[mode:mode + 6, atom, 2] = next(inputfile).split()[1:]
+                for mode in range(0, 3 * self.natom, 6):
+                    header = next(inputfile)
+                    for atom in range(self.natom):
+                        all_vibdisps[mode:mode + 6, atom, 0] = next(inputfile).split()[1:]
+                        all_vibdisps[mode:mode + 6, atom, 1] = next(inputfile).split()[1:]
+                        all_vibdisps[mode:mode + 6, atom, 2] = next(inputfile).split()[1:]
 
-            self.set_attribute('vibdisps', all_vibdisps[self.first_mode:])
+                self.set_attribute('vibdisps', all_vibdisps[self.first_mode:])
+            else:
+                # we have a single atom
+                self.set_attribute('vibdisps', numpy.array([]))
 
         # -----------
         # IR SPECTRUM
@@ -1056,15 +1274,19 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
         if line[:11] == "IR SPECTRUM":
             self.skip_lines(inputfile, ['d', 'b', 'header', 'd'])
 
-            all_vibirs = numpy.zeros((3 * self.natom,), "d")
+            if self.natom > 1:
+                all_vibirs = numpy.zeros((3 * self.natom,), "d")
 
-            line = next(inputfile)
-            while len(line) > 2:
-                num = int(line[0:4])
-                all_vibirs[num] = float(line.split()[2])
                 line = next(inputfile)
+                while len(line) > 2:
+                    num = int(line[0:4])
+                    all_vibirs[num] = float(line.split()[2])
+                    line = next(inputfile)
 
-            self.set_attribute('vibirs', all_vibirs[self.first_mode:])
+                self.set_attribute('vibirs', all_vibirs[self.first_mode:])
+            else:
+                # we have a single atom
+                self.set_attribute('vibirs', numpy.array([]))
 
         # --------------
         # RAMAN SPECTRUM
@@ -1078,16 +1300,19 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
         if line[:14] == "RAMAN SPECTRUM":
             self.skip_lines(inputfile, ['d', 'b', 'header', 'd'])
 
-            all_vibramans = numpy.zeros(3 * self.natom)
+            if self.natom > 1:
+                all_vibramans = numpy.zeros(3 * self.natom)
 
-            line = next(inputfile)
-            while len(line) > 2:
-                num = int(line[0:4])
-                all_vibramans[num] = float(line.split()[2])
                 line = next(inputfile)
+                while len(line) > 2:
+                    num = int(line[0:4])
+                    all_vibramans[num] = float(line.split()[2])
+                    line = next(inputfile)
 
-            self.set_attribute('vibramans', all_vibramans[self.first_mode:])
-
+                self.set_attribute('vibramans', all_vibramans[self.first_mode:])
+            else:
+                # we have a single atom
+                self.set_attribute('vibramans', numpy.array([]))
 
         # ORCA will print atomic charges along with the spin populations,
         #   so care must be taken about choosing the proper column.
@@ -1111,7 +1336,7 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
         #   are not printed (there is a blank line at the end).
         if line[:22] == "LOEWDIN ATOMIC CHARGES":
             self.parse_charge_section(line, inputfile, 'lowdin')
-        #CHELPG Charges            
+        #CHELPG Charges
         #--------------------------------
         #  0   C   :       0.363939
         #  1   H   :       0.025695
@@ -1445,7 +1670,7 @@ States  Energy Wavelength    D2        m2        Q2         D2+m2+Q2       D2/TO
           handle to file object
         chargestype : str
           what type of charge we're dealing with, must be one of
-          'mulliken', 'lowdin' or 'chelpg'   
+          'mulliken', 'lowdin' or 'chelpg'
         """
         has_spins = 'AND SPIN POPULATIONS' in line
 
