@@ -38,10 +38,14 @@ class Psi4(logfileparser.Logfile):
         # coefficients when printing.
         self.version_4_beta = False
 
-        # This is just used to track which part of the output we are in for Psi4,
-        # with changes triggered by ==> things like this <== (Psi3 does not have this)
+        # This is just used to track which part of the output we are in, with
+        # changes triggered by ==> things like this <==.
         self.section = None
 
+        # There are also sometimes subsections within each section, denoted
+        # with =>/<= rather than ==>/<==.
+        self.subsection = None
+
     def after_parsing(self):
 
         # Newer versions of Psi4 don't explicitly print the number of atoms.
@@ -50,8 +54,13 @@ class Psi4(logfileparser.Logfile):
                 self.set_attribute('natom', len(self.atomnos))
 
     def normalisesym(self, label):
-        """Psi4 does not require normalizing symmetry labels."""
-        return label
+        """Use standard symmetry labels instead of Psi4 labels.
+
+        To normalise:
+        (1) `App` -> `A"`
+        (2) `Ap` -> `A'`
+        """
+        return label.replace("pp", '"').replace("p", "'")
 
     # Match the number of skipped lines required based on the type of
     # gradient present (determined from the header), as otherwise the
@@ -73,17 +82,33 @@ class Psi4(logfileparser.Logfile):
 
         # Extract the version number and the version control
         # information, if it exists.
-        if "Driver" in line:
-            tokens = line.split()
+        if "An Open-Source Ab Initio Electronic Structure Package" in line:
+            version_line = next(inputfile)
+            tokens = version_line.split()
             package_version = tokens[1].split("-")[-1]
-            self.metadata["package_version"] = package_version
+            self.metadata["legacy_package_version"] = package_version
             # Keep track of early versions of Psi4.
             if "beta" in package_version:
                 self.version_4_beta = True
-        # Don't add revision information to the main package version for now.
-        if "Git:" in line:
-            tokens = line.split()
-            revision = '-'.join(tokens[2:])
+                # `beta2+` -> `0!0.beta2`
+                package_version = "0!0.{}".format(package_version)
+                if package_version[-1] == "+":
+                    # There is no good way to keep the bare plus sign around,
+                    # but this version is so old...
+                    package_version = package_version[:-1]
+            else:
+                package_version = "1!{}".format(package_version)
+            self.skip_line(inputfile, "blank")
+            line = next(inputfile)
+            if "Git:" in line:
+                tokens = line.split()
+                assert tokens[1] == "Rev"
+                revision = '-'.join(tokens[2:]).replace("{", "").replace("}", "")
+                dev_flag = "" if "dev" in package_version else ".dev"
+                package_version = "{}{}+{}".format(
+                    package_version, dev_flag, revision
+                )
+            self.metadata["package_version"] = package_version
 
         # This will automatically change the section attribute for Psi4, when encountering
         # a line that <== looks like this ==>, to whatever is in between.
@@ -92,16 +117,27 @@ class Psi4(logfileparser.Logfile):
             if self.section == "DFT Potential":
                 self.metadata["methods"].append("DFT")
 
+        # There is also the possibility of subsections.
+        if (line.strip()[:2] == "=>") and (line.strip()[-2:] == "<="):
+            self.subsection = line.strip()[3:-3]
+
         # Determine whether or not the reference wavefunction is
         # restricted, unrestricted, or restricted open-shell.
         if line.strip() == "SCF":
-            self.skip_line(inputfile, 'author list')
-            line = next(inputfile)
+            while "Reference" not in line:
+                line = next(inputfile)
             self.reference = line.split()[0]
             # Work with a complex reference as if it's real.
             if self.reference[0] == 'C':
                 self.reference = self.reference[1:]
 
+        # Parse the XC density functional
+        # => Composite Functional: B3LYP <=
+        if self.section == "DFT Potential" and "composite functional" in line.lower():
+            chomp = line.split()
+            functional = chomp[-2]
+            self.metadata["functional"] = functional
+
         #  ==> Geometry <==
         #
         #    Molecular point group: c2h
@@ -158,7 +194,8 @@ class Psi4(logfileparser.Logfile):
             # This condition discards any repeated coordinates that Psi print. For example,
             # geometry optimizations will print the coordinates at the beginning of and SCF
             # section and also at the start of the gradient calculation.
-            if len(self.atomcoords) == 0 or self.atomcoords[-1] != coords:
+            if len(self.atomcoords) == 0 \
+                or (self.atomcoords[-1] != coords and not hasattr(self, 'finite_difference')):
                 self.atomcoords.append(coords)
 
             if len(atommasses) > 0:
@@ -174,7 +211,8 @@ class Psi4(logfileparser.Logfile):
             self.set_attribute('mult', mult)
 
         # The printout for Psi4 has a more obvious trigger for the SCF parameter printout.
-        if (self.section == "Algorithm") and (line.strip() == "==> Algorithm <=="):
+        if (self.section == "Algorithm") and (line.strip() == "==> Algorithm <==") \
+            and not hasattr(self, 'finite_difference'):
 
             self.skip_line(inputfile, 'blank')
 
@@ -371,7 +409,8 @@ class Psi4(logfileparser.Logfile):
         # repulsion integrals. In that case, there are actually two convergence cycles performed,
         # one for the density-fitted algorithm and one for the exact one, and the iterations are
         # printed in two blocks separated by some set-up information.
-        if (self.section == "Iterations") and (line.strip() == "==> Iterations <=="):
+        if (self.section == "Iterations") and (line.strip() == "==> Iterations <==") \
+            and not hasattr(self, 'finite_difference'):
 
             if not hasattr(self, 'scfvalues'):
                 self.scfvalues = []
@@ -420,7 +459,8 @@ class Psi4(logfileparser.Logfile):
         #	  25Bu     1.124938
         #
         # The case is different in the trigger string.
-        if "orbital energies (a.u.)" in line.lower():
+        if ("orbital energies (a.u.)" in line.lower()  or "orbital energies [eh]" in line.lower()) \
+            and not hasattr(self, 'finite_difference'):
 
             # If this is Psi4, we will be in the appropriate section.
             assert self.section == "Post-Iterations"
@@ -504,35 +544,49 @@ class Psi4(logfileparser.Logfile):
 
         # Both Psi3 and Psi4 print the final SCF energy right after the orbital energies,
         # but the label is different. Psi4 also does DFT, and the label is also different in that case.
-        if self.section == "Post-Iterations" and "Final Energy:" in line:
+        if self.section == "Post-Iterations" and "Final Energy:" in line \
+            and not hasattr(self, 'finite_difference'):
             e = float(line.split()[3])
             if not hasattr(self, 'scfenergies'):
                 self.scfenergies = []
             self.scfenergies.append(utils.convertor(e, 'hartree', 'eV'))
 
-        #  ==> Molecular Orbitals <==
+        if self.subsection == "Energetics":
+            if "Empirical Dispersion Energy" in line:
+                dispersion = utils.convertor(float(line.split()[-1]), "hartree", "eV")
+                self.append_attribute("dispersionenergies", dispersion)
+
+        #   ==> Molecular Orbitals <==
         #
-        #                 1            2            3            4            5
+        #                     1            2            3            4            5
         #
-        #    1    0.7014827    0.7015412    0.0096801    0.0100168    0.0016438
-        #    2    0.0252630    0.0251793   -0.0037890   -0.0037346    0.0016447
-        # ...
-        #   59    0.0000133   -0.0000067    0.0000005   -0.0047455   -0.0047455
-        #   60    0.0000133    0.0000067    0.0000005    0.0047455   -0.0047455
+        # 1    H1 s0         0.1610392    0.1040990    0.0453848    0.0978665    1.0863246
+        # 2    H1 s0         0.3066996    0.0742959    0.8227318    1.3460922   -1.6429494
+        # 3    H1 s0         0.1669296    1.5494169   -0.8885631   -1.8689490    1.0473633
+        # 4    H2 s0         0.1610392   -0.1040990    0.0453848   -0.0978665   -1.0863246
+        # 5    H2 s0         0.3066996   -0.0742959    0.8227318   -1.3460922    1.6429494
+        # 6    H2 s0         0.1669296   -1.5494169   -0.8885631    1.8689490   -1.0473633
         #
-        # Ene   -11.0288198  -11.0286067  -11.0285837  -11.0174766  -11.0174764
-        # Sym            Ag           Bu           Ag           Bu           Ag
-        # Occ             2            2            2            2            2
+        #             Ene    -0.5279195    0.1235556    0.3277474    0.5523654    2.5371710
+        #             Sym            Ag          B3u           Ag          B3u          B3u
+        #             Occ             2            0            0            0            0
         #
         #
-        #                11           12           13           14           15
+        #                         6
         #
-        #    1    0.1066946    0.1012709    0.0029709    0.0120562    0.1002765
-        #    2   -0.2753689   -0.2708037   -0.0102079   -0.0329973   -0.2790813
-        # ...
+        # 1    H1 s0         1.1331221
+        # 2    H1 s0        -1.2163107
+        # 3    H1 s0         0.4695317
+        # 4    H2 s0         1.1331221
+        # 5    H2 s0        -1.2163107
+        # 6    H2 s0         0.4695317
         #
+        #            Ene     2.6515637
+        #            Sym            Ag
+        #            Occ             0
+
         if (self.section) and ("Molecular Orbitals" in self.section) \
-           and ("Molecular Orbitals" in line):
+           and ("Molecular Orbitals" in line) and not hasattr(self, 'finite_difference'):
 
             self.skip_line(inputfile, 'blank')
 
@@ -553,10 +607,13 @@ class Psi4(logfileparser.Logfile):
 
                 self.skip_line(inputfile, 'blank')
 
+                n = len(indices)
                 line = next(inputfile)
                 while line.strip():
-                    iao = int(line.split()[0])
-                    coeffs = [float(c) for c in line.split()[1:]]
+                    chomp = line.split()
+                    m = len(chomp)
+                    iao = int(chomp[0])
+                    coeffs = [float(c) for c in chomp[m - n:]]
                     for i, c in enumerate(coeffs):
                         mocoeffs[indices[i]-1].append(c)
                     line = next(inputfile)
@@ -643,7 +700,8 @@ class Psi4(logfileparser.Logfile):
         #      2    -379.77675264   -7.79e-03      1.88e-02      4.37e-03 o    2.29e-02      6.76e-03 o  ~
         #  ---------------------------------------------------------------------------------------------
         #
-        if (self.section == "Convergence Check") and line.strip() == "==> Convergence Check <==":
+        if (self.section == "Convergence Check") and line.strip() == "==> Convergence Check <==" \
+            and not hasattr(self, 'finite_difference'):
 
             if not hasattr(self, "optstatus"):
                 self.optstatus = []
@@ -718,9 +776,13 @@ class Psi4(logfileparser.Logfile):
         #
         # Properties will be evaluated at   0.000000,   0.000000,   0.000000 Bohr
         #
+        # OR
+        #
+        # Properties will be evaluated at   0.000000,   0.000000,   0.000000 [a0]
+        #
         if "Properties will be evaluated at" in line.strip():
             self.origin = numpy.array([float(x.strip(',')) for x in line.split()[-4:-1]])
-            assert line.split()[-1] == "Bohr"
+            assert line.split()[-1] in ["Bohr", "[a0]"]
             self.origin = utils.convertor(self.origin, 'bohr', 'Angstrom')
 
         # The properties section print the molecular dipole moment:
@@ -847,6 +909,8 @@ class Psi4(logfileparser.Logfile):
                 self.grads = []
             self.grads.append(gradient)
 
+        # OLD Normal mode output parser (PSI4 < 1)
+
         ## Harmonic frequencies.
 
         # -------------------------------------------------------------
@@ -917,8 +981,12 @@ class Psi4(logfileparser.Logfile):
                 _vibfreq = Psi4.parse_vibfreq(line[13:].strip())
                 assert abs(vibfreq - _vibfreq) < 1.0e-2
                 line = next(inputfile)
-                # Can't do anything with this for now.
                 assert 'Force constant:' in line
+                if not hasattr(self, "vibfconsts"):
+                    self.vibfconsts = []
+                self.vibfconsts.append(
+                    utils.convertor(float(line.split()[2]), "hartree/bohr2", "mDyne/angstrom")
+                )
                 line = next(inputfile)
                 assert 'X       Y       Z           mass' in line
                 line = next(inputfile)
@@ -938,6 +1006,81 @@ class Psi4(logfileparser.Logfile):
                 self.vibdisps.append(normal_mode_disps)
                 line = next(inputfile)
 
+        # NEW Normal mode output parser (PSI4 >= 1)
+
+        #   ==> Harmonic Vibrational Analysis <==
+        #   ...
+        #   Vibration                       7                   8                   9
+        #   ...
+        #
+        #   Vibration                       10                  11                  12
+        #   ...
+
+        if line.strip() == '==> Harmonic Vibrational Analysis <==':
+
+            vibsyms = []
+            vibfreqs = []
+            vibdisps = []
+            vibrmasses = []
+            vibfconsts = []
+
+            # Skip lines till the first Vibration block
+            while not line.strip().startswith('Vibration'):
+                line = next(inputfile)
+
+            n_modes = 0
+            # Parse all the Vibration blocks
+            while line.strip().startswith('Vibration'):
+                n = len(line.split()) - 1
+                n_modes += n
+                vibfreqs_, vibsyms_, vibdisps_, vibrmasses_, vibfconsts_ = self.parse_vibration(n, inputfile)
+                vibfreqs.extend(vibfreqs_)
+                vibsyms.extend(vibsyms_)
+                vibdisps.extend(vibdisps_)
+                vibrmasses.extend(vibrmasses_)
+                vibfconsts.extend(vibfconsts_)
+                line = next(inputfile)
+
+            # It looks like the symmetry of the normal mode may be missing 
+            # from some / most. Only include them if they are there for all
+
+            if len(vibfreqs) == n_modes:
+                self.set_attribute('vibfreqs', vibfreqs)
+
+            if len(vibsyms) == n_modes:
+                self.set_attribute('vibsyms', vibsyms)
+
+            if len(vibdisps) == n_modes:
+                self.set_attribute('vibdisps', vibdisps)
+
+            if len(vibdisps) == n_modes:
+                self.set_attribute('vibrmasses', vibrmasses)
+
+            if len(vibdisps) == n_modes:
+                self.set_attribute('vibfconsts', vibfconsts)
+
+        # If finite difference is used to compute forces (i.e. by displacing
+        # slightly all the atoms), a series of additional scf calculations is
+        # performed. Orbitals, geometries, energies, etc. for these shouln't be
+        # included in the parsed data.
+
+        if line.strip().startswith('Using finite-differences of gradients'):
+            self.set_attribute('finite_difference', True)
+
+        # This is the result of calling `print_variables()` and contains all
+        # current inner variables known to Psi4.
+        if line.strip() == "Variable Map:":
+            self.skip_line(inputfile, "d")
+            line = next(inputfile)
+            while line.strip():
+                tokens = line.split()
+                # Remove double quotation marks
+                name = " ".join(tokens[:-2])[1:-1]
+                value = float(tokens[-1])
+                if name == "CC T1 DIAGNOSTIC":
+                    self.metadata["t1_diagnostic"] = value
+                line = next(inputfile)
+
         if line[:54] == '*** Psi4 exiting successfully. Buy a developer a beer!'\
                 or line[:54] == '*** PSI4 exiting successfully. Buy a developer a beer!':
             self.metadata['success'] = True
@@ -969,6 +1112,69 @@ class Psi4(logfileparser.Logfile):
             line = next(inputfile)
         return gradient
 
+    @staticmethod
+    def parse_vibration(n, inputfile):
+
+        #   Freq [cm^-1]                1501.9533           1501.9533           1501.9533
+        #   Irrep
+        #   Reduced mass [u]              1.1820              1.1820              1.1820
+        #   Force const [mDyne/A]         1.5710              1.5710              1.5710
+        #   Turning point v=0 [a0]        0.2604              0.2604              0.2604
+        #   RMS dev v=0 [a0 u^1/2]        0.2002              0.2002              0.2002
+        #   Char temp [K]               2160.9731           2160.9731           2160.9731
+        #   ----------------------------------------------------------------------------------
+        #       1   C               -0.00  0.01  0.13   -0.00 -0.13  0.01   -0.13  0.00 -0.00
+        #       2   H                0.33 -0.03 -0.38    0.02  0.60 -0.02    0.14 -0.01 -0.32
+        #       3   H               -0.32 -0.03 -0.37   -0.01  0.60 -0.01    0.15 -0.01  0.33
+        #       4   H                0.02  0.32 -0.36    0.01  0.16 -0.34    0.60 -0.01  0.01
+        #       5   H                0.02 -0.33 -0.39    0.01  0.13  0.31    0.60  0.01  0.01
+
+        line = next(inputfile)
+        assert 'Freq' in line
+        chomp = line.split()
+        vibfreqs = [Psi4.parse_vibfreq(x) for x in chomp[-n:]]
+
+        line = next(inputfile)
+        assert 'Irrep' in line
+        chomp = line.split()
+        vibsyms = [irrep for irrep in chomp[1:]]
+
+        line = next(inputfile)
+        assert 'Reduced mass' in line
+        chomp = line.split()
+        vibrmasses = [utils.float(x) for x in chomp[3:]]
+
+        line = next(inputfile)
+        assert 'Force const' in line
+        chomp = line.split()
+        vibfconsts = [utils.float(x) for x in chomp[3:]]
+
+        line = next(inputfile)
+        assert 'Turning point' in line
+
+        line = next(inputfile)
+        assert 'RMS dev' in line
+
+        line = next(inputfile)
+        assert 'Char temp' in line
+
+        line = next(inputfile)
+        assert '---' in line
+
+        line = next(inputfile)
+        vibdisps = [ [] for i in range(n)]
+        while len(line.strip()) > 0:
+            chomp = line.split()
+            for i in range(n):
+                start = len(chomp) - (n - i) * 3
+                stop = start + 3
+                mode_disps = [float(c) for c in chomp[start:stop]]
+                vibdisps[i].append(mode_disps)
+
+            line = next(inputfile)
+
+        return vibfreqs, vibsyms, vibdisps, vibrmasses, vibfconsts
+
     @staticmethod
     def parse_vibfreq(vibfreq):
         """Imaginary frequencies are printed as '12.34i', rather than
