@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2018, the cclib development team
+# Copyright (c) 2020, the cclib development team
 #
 # This file is part of cclib (http://cclib.github.io) and is distributed under
 # the terms of the BSD 3-Clause License.
@@ -60,8 +60,13 @@ class Turbomole(logfileparser.Logfile):
         return 'Turbomole("%s")' % (self.filename)
 
     def normalisesym(self, label):
-        """Normalise the symmetries used by Turbomole."""
-        raise NotImplementedError('Now yet implemented for Turbomole.')
+        """Normalise the symmetries used by Turbomole.
+
+        The labels are standardized except for the first character being lowercase.
+        """
+        # TODO more work could be required, but we don't have any logfiles
+        # with non-C1 symmetry.
+        return label[0].upper() + label[1:]
 
     def before_parsing(self):
         self.geoopt = False # Is this a GeoOpt? Needed for SCF targets/values.
@@ -113,10 +118,12 @@ class Turbomole(logfileparser.Logfile):
         if index > -1:
             line = line[index + len(searchstr):]
             tokens = line.split()
-            self.metadata["package_version"] = tokens[0][1:].replace("-", ".")
-            # Don't add revision information to the main package version for now.
+            package_version = tokens[0][1:].replace("-", ".")
+            self.metadata["package_version"] = package_version
+            self.metadata["legacy_package_version"] = package_version
             if tokens[1] == "(":
                 revision = tokens[2]
+                self.metadata["package_version"] = "{}.r{}".format(package_version, revision)
 
         ## Atomic coordinates in job.last:
         #              +--------------------------------------------------+
@@ -150,6 +157,7 @@ class Turbomole(logfileparser.Logfile):
 
             self.append_attribute('atomcoords', atomcoords)
             self.set_attribute('atomnos', atomnos)
+            self.set_attribute('natom', len(atomcoords))
 
         # Frequency values in aoforce.out
         #        mode               7        8        9       10       11       12
@@ -168,8 +176,12 @@ class Turbomole(logfileparser.Logfile):
         #   1   c           x   0.00000  0.00001  0.00000 -0.01968 -0.04257  0.00001
         #                   y  -0.08246 -0.08792  0.02675 -0.00010  0.00000  0.17930
         #                   z   0.00001  0.00003  0.00004 -0.10350  0.11992 -0.00003
+        # ....
+        #
+        # reduced mass(g/mol)     3.315    2.518    2.061    3.358    3.191    2.323
+
         if 'NORMAL MODES and VIBRATIONAL FREQUENCIES (cm**(-1))' in line:
-            vibfreqs, vibsyms, vibirs, vibdisps = [], [], [], []
+            vibfreqs, vibsyms, vibirs, vibdisps, vibrmasses = [], [], [], [], []
             while '****  force : all done  ****' not in line:
                 if line.strip().startswith('frequency'):
                     freqs = [float(i.replace('i', '-')) for i in line.split()[1:]]
@@ -183,7 +195,7 @@ class Turbomole(logfileparser.Logfile):
                     self.skip_lines(inputfile, ['b', 'IR', 'dQIP'])
                     line = next(inputfile)
                     if line.strip().startswith('intensity (km/mol)'):
-                        irs = [self.float(f) for f in line.split()[2:]]
+                        irs = [utils.float(f) for f in line.split()[2:]]
                         vibirs.extend(irs)
 
                     self.skip_lines(inputfile, ['intensity', 'b', 'raman', 'b'])
@@ -203,12 +215,17 @@ class Turbomole(logfileparser.Logfile):
                             disps.append([x[i][j], y[i][j], z[i][j]])
                         vibdisps.append(disps)
 
+                if line.strip().startswith('reduced mass(g/mol)'):
+                    rmasses = [utils.float(f) for f in line.split()[2:]]
+                    vibrmasses.extend(rmasses)
+
                 line = next(inputfile)
 
             self.set_attribute('vibfreqs', vibfreqs)
             self.set_attribute('vibsyms', vibsyms)
             self.set_attribute('vibirs', vibirs)
             self.set_attribute('vibdisps', vibdisps)
+            self.set_attribute('vibrmasses', vibrmasses)
 
         # In this section we are parsing mocoeffs and moenergies from
         # the files like: mos, alpha and beta.
@@ -234,8 +251,8 @@ class Turbomole(logfileparser.Logfile):
             mocoeffs = []
 
             while not line.strip().startswith('$'):
-                info = re.match(".*eigenvalue=(?P<moenergy>[0-9D\.+-]{20})\s+nsaos=(?P<count>\d+).*", line)
-                eigenvalue = self.float(info.group('moenergy'))
+                info = re.match(r".*eigenvalue=(?P<moenergy>[0-9D\.+-]{20})\s+nsaos=(?P<count>\d+).*", line)
+                eigenvalue = utils.float(info.group('moenergy'))
                 orbital_energy = utils.convertor(eigenvalue, 'hartree', 'eV')
                 moenergies.append(orbital_energy)
                 single_coeffs = []
@@ -294,8 +311,8 @@ class Turbomole(logfileparser.Logfile):
         #                 :  wavefunction norm =      1.00000000000  :
         #                  ..........................................
         if 'scf convergence criterion' in line:
-            total_energy_threshold = self.float(line.split()[-1])
-            one_electron_energy_threshold = self.float(next(inputfile).split()[-1])
+            total_energy_threshold = utils.float(line.split()[-1])
+            one_electron_energy_threshold = utils.float(next(inputfile).split()[-1])
             scftargets = [total_energy_threshold, one_electron_energy_threshold]
             self.append_attribute('scftargets', scftargets)
             iter_energy = []
@@ -304,8 +321,8 @@ class Turbomole(logfileparser.Logfile):
                 if 'ITERATION  ENERGY' in line:
                     line = next(inputfile)
                     info = line.split()
-                    iter_energy.append(self.float(info[1]))
-                    iter_one_elec_energy.append(self.float(info[2]))
+                    iter_energy.append(utils.float(info[1]))
+                    iter_one_elec_energy.append(utils.float(info[2]))
                 line = next(inputfile)
 
             assert len(iter_energy) == len(iter_one_elec_energy), \
@@ -316,7 +333,7 @@ class Turbomole(logfileparser.Logfile):
             while 'total energy' not in line:
                 line = next(inputfile)
 
-            scfenergy = utils.convertor(self.float(line.split()[4]), 'hartree', 'eV')
+            scfenergy = utils.convertor(utils.float(line.split()[4]), 'hartree', 'eV')
             self.append_attribute('scfenergies', scfenergy)
 
         #  **********************************************************************
@@ -343,12 +360,14 @@ class Turbomole(logfileparser.Logfile):
         if 'C C S D F 1 2   P R O G R A M' in line:
             while 'ccsdf12 : all done' not in line:
                 if 'Final MP2 energy' in line:
-                    mp2energy = [utils.convertor(self.float(line.split()[5]), 'hartree', 'eV')]
+                    mp2energy = [utils.convertor(utils.float(line.split()[5]), 'hartree', 'eV')]
                     self.append_attribute('mpenergies', mp2energy)
 
                 if 'Final CCSD energy' in line:
-                    ccenergy = [utils.convertor(self.float(line.split()[5]), 'hartree', 'eV')]
-                    self.append_attribute('ccenergies', ccenergy)
+                    self.append_attribute(
+                        'ccenergies',
+                        utils.convertor(utils.float(line.split()[5]), 'hartree', 'eV')
+                    )
 
                 line = next(inputfile)
 
@@ -366,24 +385,24 @@ class Turbomole(logfileparser.Logfile):
                 if 'MP2-energy' in line:
                     line = next(inputfile)
                     if 'total' in line:
-                        mp2energy = [utils.convertor(self.float(line.split()[3]), 'hartree', 'eV')]
+                        mp2energy = [utils.convertor(utils.float(line.split()[3]), 'hartree', 'eV')]
                         self.append_attribute('mpenergies', mp2energy)
                 line = next(inputfile)
 
-    def deleting_modes(self, vibfreqs, vibdisps, vibirs):
+    def deleting_modes(self, vibfreqs, vibdisps, vibirs, vibrmasses):
         """Deleting frequencies relating to translations or rotations"""
         i = 0
         while i < len(vibfreqs):
             if vibfreqs[i] == 0.0:
                 # Deleting frequencies that have value 0 since they
                 # do not correspond to vibrations.
-                del vibfreqs[i], vibdisps[i], vibirs[i]
+                del vibfreqs[i], vibdisps[i], vibirs[i], vibrmasses[i]
                 i -= 1
             i += 1
 
     def after_parsing(self):
         if hasattr(self, 'vibfreqs'):
-            self.deleting_modes(self.vibfreqs, self.vibdisps, self.vibirs)
+            self.deleting_modes(self.vibfreqs, self.vibdisps, self.vibirs, self.vibrmasses)
 
 
 class OldTurbomole(logfileparser.Logfile):
@@ -810,7 +829,7 @@ class OldTurbomole(logfileparser.Logfile):
 
             temp=line.replace("i","-").split()
 
-            freqs = [self.float(f) for f in temp[1:]]
+            freqs = [utils.float(f) for f in temp[1:]]
             self.vibfreqs.extend(freqs)
                     
             line=inputfile.next()
@@ -825,7 +844,7 @@ class OldTurbomole(logfileparser.Logfile):
             line=inputfile.next()
 
             temp=line.split()
-            irs = [self.float(f) for f in temp[2:]]
+            irs = [utils.float(f) for f in temp[2:]]
             self.vibirs.extend(irs)
 
             line=inputfile.next()
