@@ -9,11 +9,12 @@
 
 import sys
 import numpy
+import periodictable
+import scipy.spatial.transform
 
 
-# Define for any Python version <= 3.3,
 # See https://github.com/kachayev/fn.py/commit/391824c43fb388e0eca94e568ff62cc35b543ecb
-if sys.version_info.major == 2 or sys.version_info.minor <= 3:
+if sys.version_info <= (3, 3):
     import operator
     def accumulate(iterable, func=operator.add):
         """Return running totals"""
@@ -32,6 +33,20 @@ else:
     from itertools import accumulate
 
 
+def find_package(package):
+    """Check if a package exists without importing it.
+
+    Derived from https://stackoverflow.com/a/14050282
+    """
+    if sys.version_info.major == 2:
+        import pkgutil
+        return pkgutil.find_loader(package) is not None
+    else:
+        import importlib
+        module_spec = importlib.util.find_spec(package)
+        return module_spec is not None and module_spec.loader is not None
+
+
 def symmetrize(m, use_triangle='lower'):
     """Symmetrize a square NumPy array by reflecting one triangular
     section across the diagonal to the other.
@@ -59,9 +74,27 @@ def symmetrize(m, use_triangle='lower'):
     return ms
 
 
+_BUILTIN_FLOAT = float
+
+
+def float(number):
+    """Convert a string to a float.
+
+    This method should perform certain checks that are specific to cclib,
+    including avoiding the problem with Ds instead of Es in scientific notation.
+    Another point is converting string signifying numerical problems (*****)
+    to something we can manage (Numpy's NaN).
+    """
+
+    if list(set(number)) == ['*']:
+        return numpy.nan
+
+    return _BUILTIN_FLOAT(number.replace("D", "E"))
+
+
 def convertor(value, fromunits, tounits):
     """Convert from one set of units to another.
-    
+
     Sources:
         NIST 2010 CODATA (http://physics.nist.gov/cuu/Constants/index.html)
         Documentation of GAMESS-US or other programs as noted
@@ -119,39 +152,78 @@ def convertor(value, fromunits, tounits):
         "ebohr4_to_Debye.ang3": lambda x: x * 0.3766479268,
         "ebohr5_to_Debye.ang4": lambda x: x * 0.1993134985,
 
+        "hartree/bohr2_to_mDyne/angstrom": lambda x: x * 8.23872350 / 0.5291772109
     }
 
     return _convertor["%s_to_%s" % (fromunits, tounits)](value)
 
+def _get_rmat_from_vecs(a, b):
+    """Get rotation matrix from two 3D vectors, a and b
+    Args:
+       a (np.ndaray): 3d vector with shape (3,0)
+       b (np.ndaray): 3d vector with shape (3,0)
+    Returns:
+       np.ndarray
+    """
+    a_ = (a / numpy.linalg.norm(a, 2))
+    b_ = (b / numpy.linalg.norm(b, 2))
+    v = numpy.cross(a_, b_)
+    s = numpy.linalg.norm(v, 2)
+    c = numpy.dot(a_, b_)
+    # skew-symmetric cross product of v
+    vx = numpy.array([[0, -v[2], v[1]],
+                    [v[2], 0, -v[0]],
+                    [-v[1], v[0], 0]])
+    rmat = numpy.identity(3) + vx + numpy.matmul(vx, vx) * ((1-c)/s**2)
+    return rmat
+
+def get_rotation(a, b):
+    """Get rotation part for transforming a to b, where a and b are same positions with different orientations
+    If one atom positions, i.e (1,3) shape array, are given, it returns identify transformation
+
+    Args:
+        a (np.ndarray): positions with shape(N,3)
+        b (np.ndarray): positions with shape(N,3)
+    Returns:
+        scipy.spatial.transform.Rotation
+    """
+    assert a.shape == b.shape
+    if a.shape[0] == 1:
+        return scipy.spatial.transform.Rotation.from_euler('xyz', [0,0,0])
+    # remove translation part
+    a_ = a - a[0]
+    b_ = b - b[0]
+    if hasattr(scipy.spatial.transform.Rotation, "align_vectors"):
+        r, _ = scipy.spatial.transform.Rotation.align_vectors(b_, a_)
+    else:
+        if numpy.linalg.matrix_rank(a_) == 1:
+            # in the case of linear molecule, e.g. O2, C2H2
+            idx = numpy.argmax(numpy.linalg.norm(a_, ord=2, axis=1))
+            rmat = _get_rmat_from_vecs(a_[idx], b_[idx])
+            r = scipy.spatial.transform.Rotation.from_dcm(rmat)
+        else:
+            # scipy.spatial.transform.Rotation.match_vectors has bug
+            # Kabsch Algorithm
+            cov = numpy.dot(b_.T, a_)
+            V, S, W = numpy.linalg.svd(cov)
+            if ((numpy.linalg.det(V) * numpy.linalg.det(W))< 0.0):
+                S[-1] = -S[-1]
+                V[:,-1] = -V[:,-1]
+            rmat = numpy.dot(V, W)
+            r = scipy.spatial.transform.Rotation.from_dcm(rmat)
+    return r
 
 class PeriodicTable(object):
     """Allows conversion between element name and atomic no."""
 
     def __init__(self):
-        self.element = [
-            None,
-            'H', 'He',
-            'Li', 'Be',
-            'B', 'C', 'N', 'O', 'F', 'Ne',
-            'Na', 'Mg',
-            'Al', 'Si', 'P', 'S', 'Cl', 'Ar',
-            'K', 'Ca',
-            'Sc', 'Ti', 'V', 'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn',
-            'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr',
-            'Rb', 'Sr',
-            'Y', 'Zr', 'Nb', 'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd',
-            'In', 'Sn', 'Sb', 'Te', 'I', 'Xe',
-            'Cs', 'Ba',
-            'La', 'Ce', 'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb',
-            'Lu', 'Hf', 'Ta', 'W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg',
-            'Tl', 'Pb', 'Bi', 'Po', 'At', 'Rn',
-            'Fr', 'Ra',
-            'Ac', 'Th', 'Pa', 'U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 'Es', 'Fm', 'Md', 'No',
-            'Lr', 'Rf', 'Db', 'Sg', 'Bh', 'Hs', 'Mt', 'Ds', 'Rg', 'Cn',
-            'Uut', 'Fl', 'Uup', 'Lv', 'Uus', 'Uuo']
+        self.element = [None]
         self.number = {}
-        for i in range(1, len(self.element)):
-            self.number[self.element[i]] = i
+        
+        for e in periodictable.elements:
+            if e.symbol != 'n':
+                self.element.append(e.symbol)
+                self.number[e.symbol] = e.number
 
 
 class WidthSplitter:
