@@ -12,13 +12,17 @@ import unittest
 
 sys.path.insert(1, "method")
 
+from .method.testbader import *
 from .method.testcda import *
+from .method.testddec import *
+from .method.testelectrons import *
+from .method.testhirshfeld import *
 from .method.testmbo import *
+from .method.testmoments import *
 from .method.testnuclear import *
 from .method.testorbitals import *
 from .method.testpopulation import *
-if sys.version_info[0] == 2:
-    from .method.testvolume import *
+from .method.testvolume import *
 
 
 if __name__ == "__main__":
